@@ -1,20 +1,17 @@
-#include "gamefile.h"
+/* This program is free software. It comes without any warranty, to
+ * the extent permitted by applicable law. You can redistribute it
+ * and/or modify it under the terms of the Do What The Fuck You Want
+ * To Public License, Version 2, as published by Sam Hocevar. See
+ * http://www.wtfpl.net/ for more details. */
 
-gameFile::gameFile(const int& t_updated, const std::string& t_id, const std::string& t_name, const std::string& t_path, const std::string& t_size, const unsigned int& t_language, const unsigned int& t_platform, const int& t_silent)
-{
-    this->updated = t_updated;
-    this->id = t_id;
-    this->name = t_name;
-    this->path = t_path;
-    this->size = t_size;
-    this->platform = t_platform;
-    this->language = t_language;
-    this->silent = t_silent;
-}
+#include "gamefile.h"
 
 gameFile::gameFile()
 {
-    //ctor
+    this->platform = GlobalConstants::PLATFORM_WINDOWS;
+    this->language = GlobalConstants::LANGUAGE_EN;
+    this->silent = 0;
+    this->type = 0;
 }
 
 gameFile::~gameFile()
@@ -44,6 +41,8 @@ Json::Value gameFile::getAsJson()
     json["platform"] = this->platform;
     json["language"] = this->language;
     json["silent"] = this->silent;
+    json["gamename"] = this->gamename;
+    json["type"] = this->type;
 
     return json;
 }
